/***********************************************************************
 * YoLink API class
 *
 * Copyright (c) 2022 David Kerr
 *
 * Based on documentation at http://doc.yosmart.com
 *
 * Manages login with User Access Credentials (UAC), obtaining list
 * of devices from YoLink servers, getting and setting values, and
 * subscribing to alerts and messages with MQTT
 */
import { YoLinkHomebridgePlatform } from './platform';
export declare class YoLinkAPI {
    private readonly platform;
    private yolinkTokens;
    private yolinkHomeId;
    private yolinkLoggedIn;
    private accessSemaphore;
    private accessTokenRefreshAt;
    private accesstokenHeartbeatAt;
    private accessTokenExpireTime;
    private accessTokenHeartbeat;
    private mqttTokenExpireTime;
    private mqttHost;
    private mqttClient;
    constructor(platform: YoLinkHomebridgePlatform);
    /*********************************************************************
     * _apiError
     *
     */
    _apiError(platform: YoLinkHomebridgePlatform, method: any, response: any, budp: any): boolean;
    /*********************************************************************
     * login
     *
     */
    login(platform: YoLinkHomebridgePlatform): Promise<boolean>;
    /*********************************************************************
     * getAccessToken
     *
     */
    getAccessToken(platform: YoLinkHomebridgePlatform): Promise<string | null>;
    /*********************************************************************
     * getDeviceList
     *
     */
    getDeviceList(platform: YoLinkHomebridgePlatform): Promise<any>;
    /*********************************************************************
     * getHomeId
     *
     */
    getHomeId(platform: YoLinkHomebridgePlatform): Promise<string | null>;
    /*********************************************************************
     * getDeviceState
     *
     */
    getDeviceState(platform: YoLinkHomebridgePlatform, device: any): Promise<any>;
    /*********************************************************************
     * setDeviceState
     *
     */
    setDeviceState(platform: YoLinkHomebridgePlatform, device: any, state: any): Promise<any>;
    /*********************************************************************
     * mqtt
     *
     * Open a MQTT session with YoLink API server to recieve update
     * messages from YoLink for each device.
     *
     * This has not been thoroughly tested.  How it behaves on roaming (IP
     * address changes) or temporary disconnects/reconnects has not been
     * full tested.  Info logging is enabled to capture as much information
     * as possible for events like this. Please report bugs.
     */
    mqtt(platform: YoLinkHomebridgePlatform, msgCallback: any): Promise<void>;
}
//# sourceMappingURL=yolinkAPI.d.ts.map